import '../Stream.dart';
import '../Tag.dart';

class IntArrayTag extends Tag {
  final List<int> value = [];
  IntArrayTag();

  IntArrayTag._(List<int> value) {
    this.value.addAll(value);
  }

  static IntArrayTag valueOf(List<int> value) {
    return IntArrayTag._(value);
  }

  @override
  void readValue(ByteLayer data) {
    int count = data.readInt();
    for (int i = 0; i < count; i++) {
      value.add(data.readInt());
    }
  }

  @override
  void writeValue(ByteLayer data) {
    data.writeInt(value.length);
    for (int i in value) {
      data.writeInt(i);
    }
  }

  @override
  TagType getTagType() {
    return TagType.IntArray;
  }

  @override
  void setValue(val) {}

  @override
  getValue() {}

  @override
  void prettyPrint(int indent, bool recurse) {
    String array = "";
    for (int b in value) {
      array += "$b, ";
    }
    array = array.substring(0, array.length - 2);

    print(
        "${"".padLeft(indent, '\t')}${Tag.getCanonicalName(getTagType())}: [$array]");
  }

  @override
  void writeStringifiedValue(StringBuilder builder, int indent, bool isList) {
    builder.append(
        "${isList ? "".padLeft(indent, '\t') : ""}[I; ${value.join('I, ')}I]");
  }

  @override
  void readStringifiedValue(StringReader reader) {
    reader.expect("[");
    reader.expect("I");
    reader.expect(";");
    while (reader.peek() != "]") {
      value.add(int.parse(reader.readNumber()));

      if (reader.peek() == ",") reader.next();
    }
    reader.expect("]");
  }
}
