import '../Stream.dart';
import '../Tag.dart';

class StringTag extends Tag {
  String value = "";
  StringTag();

  StringTag._(String str) {
    value = str;
  }

  static StringTag valueOf(String str) {
    return StringTag._(str);
  }

  @override
  void readValue(ByteLayer data) {
    value = data.readString();
  }

  @override
  void writeValue(ByteLayer data) {
    data.writeString(value);
  }

  @override
  TagType getTagType() {
    return TagType.String;
  }

  @override
  void setValue(val) {
    if (val is String) value = val;
  }

  @override
  getValue() {
    return value;
  }

  @override
  void prettyPrint(int indent, bool recurse) {
    print(
        "${"".padLeft(indent, '\t')}${Tag.getCanonicalName(getTagType())}: $value");
  }

  @override
  void writeStringifiedValue(StringBuilder builder, int indent, bool isList) {
    if (shouldQuote(value))
      builder.append("${isList ? "".padLeft(indent, '\t') : ""}\"${value}\"");
    else
      builder.append("${isList ? "".padLeft(indent, '\t') : ""}${value}");
  }

  @override
  void readStringifiedValue(StringReader reader) {
    String str = reader.readString();
    value = str;
  }
}
